/*
 * Copyright 2004-2007 H2 Group. Licensed under the H2 License, Version 1.0 (http://h2database.com/html/license.html).
 */

CREATE TABLE CHANNEL(TITLE VARCHAR, LINK VARCHAR, DESC VARCHAR,
    LANGUAGE VARCHAR, PUB TIMESTAMP, LAST TIMESTAMP, AUTHOR VARCHAR);

INSERT INTO CHANNEL VALUES('H2 Database Engine' ,
    'http://www.h2database.com', 'H2 Database Engine', 'en-us', NOW(), NOW(), 'Thomas Mueller');

CREATE TABLE ITEM(ID INT PRIMARY KEY, TITLE VARCHAR, ISSUED TIMESTAMP, DESC VARCHAR);

INSERT INTO ITEM VALUES(26,
'New version available: 1.0 / 2007-08-02', '2007-08-02 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
(You may have to click ''Refresh'').

<br />
<b>Changes and new functionality:</b>
<ul>
</li><li>H2 databases can now be accessed using the PostgreSQL ODBC driver.
</li><li>The old H2 ODBC driver has been removed.
</li><li>Function REGEXP_REPLACE and regular expression LIKE: REGEXP.
</li><li>CREATE TABLE ... AS SELECT now needs less memory.
</li><li>The per session undo log can now be disabled.
</li><li>Referential integrity can now be disabled.
</li><li>To avoid memory problems when using large transactions,
    h2.defaultMaxMemoryUndo is now 50000.    
</li><li>DEFAULT_MAX_LENGTH_INPLACE_LOB is now 1024.
</li><li>The cache size is now measured in KB.
</li><li>Optimization for NOT, boolean columns, and certain joins. 
</li><li>Part of the documentation has been translated to Japanese.
</li><li>The error messages (src/main/org/h2/res/_*.*) can now be translated.
</li><li>A new tool to help translation has been implemented
</li><li>The SysTray tool has been removed.
</ul>
<b>Bugfixes:</b>
<ul>
</li><li>Running out of memory while inserting could corrupt the database.
</li><li>Some Unicode characters where not supported as identifiers.
</li><li>H2 Console: The shutdown button works again.
</li><li>LOBs were not backed up using the BACKUP statement or tool when 
    h2.lobFilesInDirectories was enabled. 
</li><li>Calculation of cache memory usage has been improved.
</li><li>In some situations records were released from the cache too late.
</li><li>Documentation: the source code in ''Compacting a Database'' was incorrect.
</li><li>Result set in the H2 Console can now be modified again.
</li><li>Views using UNION did not work correctly.
</li><li>Function tables did not work with views and EXPLAIN.
</li>
</ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(25,
'New version available: 1.0 / 2007-07-12', '2007-07-12 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>The H2 Console can run as a standalone web application.
</li><li>The default database name is now jdbc:h2:~/test.
</li><li>The new view implementation is now enabled by default. 
</li><li>The Polish translation is available. Thanks a lot to Tomek!
</li><li>Support for the system property baseDir.
</li><li>Improved PostgreSQL compatibility.
</li><li>New column ID in INFORMATION_SCHEMA tables.
</li><li>PreparedStatement.getMetaData is now implemented.
</li><li>New setting h2.allowBigDecimalExtensions.
</li><li>The SQL statement SET ASSERT has been deprecated.
</li><li>The trace level for JdbcDataSourceFactory is now ERROR.
</li><li>Referential integrity violation: Two SQL states are now used.
</li><li>DatabaseEventListener.exceptionThrown is changed.
</li><li>The catalog name can now be used in queries.
</li><li>The default result set type is now FETCH_FORWARD.
</li>
</ul>
<b>Bugfixes:</b>
<ul>
</li><li>Views did not work in some cases. 
</li><li>LIKE ESCAPE did not work correctly in some cases.
</li><li>In some situations, large objects were deleted.
</li><li>CREATE TABLE AS SELECT .. UNION .. did not work.
</li><li>Sometimes temp files were not deleted.
</li><li>PooledConnection.getConnection is now faster.
</li><li>Deleting databases in the root directory now works.
</li><li>Windows service: the CLASSPATH was not included.
</li><li>For READ_COMMITTED, when the multi-threaded 
    kernel is enabled, read locks are now acquired but released 
    immediately after a query.
</li>
</ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(24,
'New version available: 1.0 / 2007-06-17', '2007-06-17 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>New Console starter application uses the JDK 1.6 system tray 
functionality if available, or a simple AWT frame for other platforms. 
To try it out, execute java org.h2.tools.Console. Feedback is welcome. 
This console starter application is not the default yet, 
but the plan is to remove the SysTray tool in the future.
</li><li>If a Reader or InputStream of a LOB is not closed, 
the LOB can not be deleted (embedded mode only). The exception is 
typically ''Error while renaming file''). As a workaround, set the 
system property ''h2.lobCloseBetweenReads'' to true to close the LOB 
files between read operations. However this slows down reading.
</li><li>Views support has been partially re-implemented. 
Views are up to 6 times faster. Compared to regular queries, only 
20% overhead. Because this is a bigger change, it is not enabled 
by default. To enable it, set the system property ''h2.indexNew'' 
to true (java -Dh2.indexNew=true ..., or in source code 
Constants.INDEX_NEW = true). If no problems are found, this will 
be enabled by default in the next release.
</li><li>Support for the data type CHAR. The difference to 
VARCHAR is: trailing spaces are ignored. This data type is supported 
for compatibility with other databases and older applications.
</li><li>Compatibility: Support for the data type notation 
CHARACTER VARYING.
</li><li>File names starting with ~ are now in the user directory 
(Java system property user.home)
</li><li>Can now ORDER BY -1 (meaning order by first column, 
descending), and ORDER BY ? (parameterized column number).
</li><li>Linked tables can now emit UPDATE statements if 
''EMIT UPDATES'' is specified in the CREATE LINKED TABLE statement. 
So far, updating a row always deleted the old row and then inserted 
the new row.
</li><li>New functions LEAST and GREATEST to get the smallest or 
largest value from a list.
</li><li>For most IOExceptions now the file name is included in 
the error message.
</li><li>New method Csv.write(Writer writer, ResultSet rs)
</li><li>The table id (important for LOB files) is now included in 
INFORMATION_SCHEMA.TABLES.
</li><li>The aggregate function COUNT(...) now returns a long 
instead of an int.
</li>
</ul>
<b>Bugfixes:</b>
<ul>
<li>In the last release, the H2 Console opened two connection 
when logging into a database, and only closed one connection 
when logging out. Fixed.
</li><li>In many situations, views did not use an index if they 
could have. Fixed. Also the explain plan for views works now.
</li><li>Server mode: the server stack trace was included in 
SQLException messages. Fixed.
</li><li>Databases with invalid linked tables (for example, because 
the target database is not accessible) can now be opened. Old 
table links don''t work however.
</li><li>In INSERT and MERGE statements, each column may 
only be specified once now.
</li><li>A java.util.Date object is now converted to a TIMESTAMP 
in the JDBC API. Previously it was converted to a DATE.
</li><li>After calling SHUTDOWN and closing the connection and 
a superfluous error message appeared in the trace file. Fixed.
</li><li>When using DISTINCT, ORDER BY a function works now 
as long as it is in the column list.
</li><li>The ''ordering'' of data types was not always correct, 
for example an operation involving REAL and DOUBLE produced 
a result of type REAL. Fixed.
</li><li>CSV tool: If the same instance was used for reading 
and writing, the tool wrote the column names twice. Fixed.
</li><li>There was a small memory leak in the trace module. 
One object per opened connection was kept in a hash map.
</li>
</ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(23,
'New version available: 1.0 / 2007-04-29', '2007-04-29 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>New function TABLE to define ad-hoc (temporary) tables in a query. 
    This also solves problems with variable-size IN(...) queries: 
    instead of SELECT * FROM TEST WHERE ID IN(?, ?, ...) you can now write:
    SELECT * FROM TABLE(ID INT=?) X, TEST WHERE X.ID=TEST.ID
    In this case, the index is used.
</li><li>New data type ARRAY. Actually it was there before, but is now documented 
    and better tested (however it must still be considered experimental).
    The java.sql.Array implementation is incomplete, but setObject(1, new Object[]{...})
    and getObject(..) can be used. New functions ARRAY_GET and ARRAY_LENGTH.
</li><li>Autocomplete in the Console application: now the result frame scrolls to the top when the list is updated.
</li><li>SimpleResultSet now has some basic data type conversion features.
</li><li>The BACKUP command is better tested and documented.
    This means hot backup (online backup) is now possible.
</li><li>The old ''Backup'' tool is now called ''Script'' (as the SQL statement).
</li><li>There are new ''Backup'' and ''Restore'' tools that work with database files directly.
</li><li>The new function LINK_SCHEMA simplifies linking all tables of a schema.
</li><li>SCRIPT DROP now also drops aliases (Java functions) if they exist.
</li><li>For encrypted databases, the trace option can no longer be enabled manually by creating a file.
</li><li>For linked tables, NULL in the unique key is now supported.
</li><li>For read-only databases, temp files are now created in the default temp directory instead
    of the database directory.
</li><li>CSVWRITE now returns the number of rows written.
</li><li>The Portuguese (Europe) translation is available. Thanks a lot to Antonio Casqueiro!
</li><li>The error message for invalid views has been improved (the root cause is included in the message now).
</li><li>SQLException.getCause of the now works for JDK 1.4 and higher.
</li>
</ul>
<b>Bugfixes:</b>
<ul>
<li>Unnamed private in-memory database (jdbc:h2:mem:) were not ''private'' as documented. Fixed.
</li><li>GROUP BY expressions did not work correctly in subqueries. Fixed.
</li><li>When using JDK 1.5 or later, and switching on h2.lobFilesInDirectories, 
    the performance for creating LOBs was bad. This has been fixed, however
    creating lots of LOBs it is still faster when the setting is switched off.
</li><li>A problem with multiple unnamed dynamic tables (FROM (SELECT...)) has been fixed.
</li><li>Appending ''Z'' to a timestamp did not have an effect. Now it is interpreted as +00:00 (GMT).
</li><li>The complete syntax for referential and check constraints is now supported 
    when written as part of the column definition, behind PRIMARY KEY.
</li><li>CASE WHEN ... returned the wrong result when the condition evaluated to NULL.
</li><li>Sending CLOB data was slow in some systems when using the server version. Fixed.
</li><li>The data type of NULLIF was NULL if the first expression was a column. Now the data type is set correctly.
</li><li>Indexes (and other related objects) for local temporary tables where not dropped 
    when the session was closed. Fixed.
</li><li>ALTER TABLE did not work for tables with computed columns.
</li><li>If the index file was deleted, an error was logged in the .trace.db file. This is no longer done.
</li><li>IN(SELECT ...) was not working correctly if the subquery returned a NULL value. Fixed.
</li><li>DROP ALL OBJECTS did not drop constants.
</li><li>DROP ALL OBJECTS dropped the role PUBLIC, which was wrong. Fixed.
</li><li>CASE was parsed as a function if the expression was in (). Fixed.
</li><li>When ORDER BY was used together with DISTINCT, it was required to type the column
    name exactly in the select list and the order list exactly in the same way. 
    This is not required any longer.
</li>
</ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(22,
'New version available: 1.0 / 2007-03-04', '2007-03-04 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>System sequences (automatically created sequences for IDENTITY or AUTO_INCREMENT columns) are now
    random (UUIDs) to avoid clashes when merging databases using RUNSCRIPT.
</li><li>Now the server tool (org.h2.tools.Server) terminates with an exit code if a problem occured.
</li><li>The JDBC driver is now loaded if the JdbcDataSource class is loaded.
</li><li>After renaming a user the password becomes invalid. This is now documented.
</li><li>Truncating a table is now allowed if the table references another table 
    (but still not allowed if the table is references by another table).
</li>
</ul>
<b>Bugfixes:</b>
<ul>
<li>The precision for linked tables was not correct for some data types, for example VARCHAR. Fixed.
</li><li>Many problems and bugs in the XA support (package javax.sql) have been fixed.
</li><li>ORDER BY picked the wrong column if the same column name (but with a different table name) 
    was used twice in the select list.
</li><li>When a subquery was used in the select list of a query, and GROUP BY was used at the same time,
    a NullPointerException could occur. Fixed.
</li><li>ORDER BY did not work when DISTINCT was used at the same time in some situations. Fixed.
</li><li>When using IN(...) on a case insensitive column (VARCHAR_IGNORECASE), 
    an incorrect optimization was made and the result was wrong sometimes.
</li><li>XAResource.recover didn''t work. Fixed. 
</li><li>XAResource.recover did throw an exception with the code XAER_OUTSIDE if there
    was no connection. Now the code is XAER_RMERR.  
</li><li>SCRIPT did not work correctly with BLOB or CLOB data. Fixed.
</li><li>BACKUP TO ''test.zip'' now works with encrypted databases and CLOB and BLOB data.
</li><li>The function CASE WHEN ... didn''t convert the returned value to the same data type,
    resulting in unexpected behavior in many cases. Fixed.
</li>
</ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(21,
'New version available: 1.0 / 2007-01-30', '2007-01-30 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>Experimental online backup feature using the SQL statement BACKUP TO ''fileName''.</li>
<li>Temporary files are now deleted earlier.</li>
<li>Benchmark: Added a multi-client test case, BenchB (similar to TPC-B).</li>
<li>The Console is now translated to Hungarian thanks to Andras Hideg, and to Indonesian thanks to Joko Yuliantoro.</li>
<li>Can now use UUID columns as generated key values.</li>
</ul>
<b>Bugfixes:</b>
<ul>
<li>In some situations, it was possible that SUM threw a class cast exception.</li>
<li>Compatibility: SCHEMA_NAME.SEQUENCE_NAME.NEXTVAL now works as expected.</li>
<li>XAConnection: A NullPointerException was thrown if addConnectionEventListener was called before opening the connection.</li>
<li>In case the result set of a subquery was re-used, an exception was throws if the subquery result did not fit in memory.</li>
<li>The command "drop all objects delete files" did not work on Linux.</li>
<li>DataSource: improved exception when setting the URL to an empty string.</li>
<li>Parsing of LIKE .. ESCAPE did not stop at the expected point.</li>
<li>Forum subscriptions (the emails sent from the forum) now work.</li>
</ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(20,
'New version available: 1.0 / 2007-01-17', '2007-01-17 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>The Console is now translated to Japanese thanks to 
    IKEMOTO, Masahiro (ikeyan (at) arizona (dot) ne (dot) jp).
</li><li>The database engine can now be compiled with JDK 1.3 using ant codeswitch. 
    There are still some limitations, and the ant script to build the jar does not work yet.
</li><li>SCRIPT NODATA now writes the row count for each table.
</li><li>Timestamps with timezone information (Z or +/-hh:mm) and dates before year 1 
    can now be parsed. However dates before year 1 are not formatted correctly.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>Fixed a problem where data in the log file was not written to the data file 
    (recovery failure) after a crash, if an index was deleted previously.
</li><li>Setting the collation (SET COLLATOR) was very slow on some systems (up to 24 seconds).
</li><li>Selecting a column using the syntax schemaName.tableName.columnName did not work in all cases.
</li><li>When stopping the TCP server from an application and immediately afterwards starting 
    it again using a different TCP password, an exception was thrown sometimes.
</li><li>Now PreparedStatement.setBigDecimal(..) can only be called with an object of 
    type java.math.BigDecimal. Derived classes are not allowed any more. Many thanks to 
    Maciej Wegorkiewicz for finding this problem.
</li><li>It was possible to manipulate values in the byte array after calling PreparedStatement.setBytes, 
    and this could lead to problems if the same byte array was used again. Now the byte array 
    is copied if required.
</li><li>Date, time and timestamp objects were cloned in cases where it was not required. Fixed. 
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(19,
'New version available: 1.0 / 2007-01-02', '2007-01-02 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>H2 is now available in Maven. The groupId is com.h2database,     the 
    artifactId is h2 and the version 1.0.20061217 (the new version will be 
    available in a few days). To create the maven artifacts yourself, use 
    ''ant mavenUploadLocal'' and ''ant mavenBuildCentral''.
</li><li>Many settings are now initialized from system properties and can be 
    changed on the command line without having recompile the database. 
    See Advances / Settings Read from System Properties.
</li><li>The (relative or absolute) directory where the script files are stored 
    or read can now be changed using the system property h2.scriptDirectory
</li><li>Client trace files now created in the directory ''trace.db'' and no 
    longer the application directory. This can be changed using the system 
    property h2.clientTraceDirectory.
</li><li>Build: Now using ant-build.properties. The JDK is automatically updated 
    when using ant codeswitch...
</li><li>Cluster: Now the server can detect if a query is read-only, and in this 
    case the result is only read from the first cluster node. However, there 
    is currently no load balancing made to avoid problems with transactions 
    / locking.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>If a CLOB or BLOB was deleted in a transaction and the database crashed 
    before the transaction was committed or rolled back, the object was lost if 
    it was large. Fixed.
</li><li>Prepared statements with non-constant functions such as 
    CURRENT_TIMESTAMP() did not get re-evaluated if the result of the 
    function changed. Fixed.
</li><li>In some situations the log file got corrupt if the process was terminated 
    while the database was opening.
</li><li>Using ;RECOVER=1 in the database URL threw a syntax exception. Fixed.
</li><li>It was possible to drop the sequence of a temporary tables with DROP 
    ALL OBJECTS, resulting in a null pointer exception afterwards.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(18,
'New version available: 1.0 / 2006-12-17', '2006-12-17 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>Large BLOB and CLOB support for the server and the cluster mode. 
    Larger objects will temporarily be buffered on the client side.
</li><li>Can be compiled with JDK 1.6.
     However, only very few of the JDBC 4.0 features are implemented so far.
</li><li>Table aliases are now supported in DELETE and UPDATE. 
    Example: DELETE FROM TEST T0.
</li><li>The RunScript tool can now include other files using a new syntax: 
    @INCLUDE fileName. This is only required for server and cluster modes.
    It was already possible to use embedded RUNSCRIPT statements, 
    but those are always executed locally.
</li><li>When the database URL contains ;RECOVERY=TRUE then 
    the index file is now deleted if it was not closed before.
</li><li>Deleting old temp files now uses a phantom reference queue. 
    Generally, temp files should now be deleted earlier.
</li><li>Opening a large database is now much faster 
    even when using the default log mode (LOG=1), 
    if the database was previously closed.
</li><li>Support for indexed parameters in PreparedStatements: 
    UPDATE TEST SET NAME = ?2 WHERE ID = ?1
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>Unfortunately, the Hibernate dialect has changed due to a change 
    in the meta data in the last release (INFORMATION_SCHEMA.SEQUENCES).
</li><li>String.toUpperCase and toLowerCase can not be used to parse commands.
    Now using toUpperCase(Locale.ENGLISH) or Character.toUpperCase(..).
</li><li>The scale of a NUMERIC(1) column is now 0. It used to be 32767.
</li><li>PreparedStatement.setObject(x, y, Types.OTHER) does now 
    serialize the object in every case (even for Integer).
</li><li>EXISTS subqueries with parameters were not re-evaluated 
    when the prepared statement was reused. This could lead to incorrect results.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(17,
'New version available: 1.0 / 2006-12-03', '2006-12-03 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>New connection time setting CACHE_TYPE=TQ to use the 2Q page replacement 
    algorithm. 2Q should be more resistant to table scan.
</li><li>Improved performance for CREATE INDEX (about 10%).
</li><li>Optimization: The left and right side of a AND and OR conditions are now 
    ordered by the expected cost.
</li><li>Java functions returning a result set (such as CSVREAD) now can be used in 
    a SELECT statement like a table. The behaviour has been changed: Now first 
    call contains the values if set, but the connection URL is different 
    (jdbc:columnlist:connection instead of jdbc:default:connection).
</li><li>The service wrapper is now included in the default installation and documented.
</li><li>New system function SESSION_ID().
</li><li>Mixing certain data types in an operation, for example VARCHAR and 
    TIMESTAMP, now converts both expressions to TIMESTAMP.
</li><li>Change behaviour: If both sides of a comparison are parameters with unknown 
    data type, then an exception is thrown now. The same happens for UNION 
    SELECT if both columns are parameters.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>There was a bug in the database encryption algorithm. 
    Pattern of repeated encrypted bytes where generated for empty blocks in the 
    file. If you have an encrypted database, you will need to decrypt the database 
    using the org.h2.tools.ChangePassword (using the old database engine), and 
    encrypt the database using the new engine. Alternatively, you can use the 
    Backup and RunScript tools or the SQL commands SCRIPT and RUNSCRIPT.
</li><li>Deeply nested views where slow to execute, because the query was parsed 
    many times. Fixed.
</li><li>The SQL statement COMMENT did not work as expected. If you already have 
    comments in the database, it is recommended to backup and restore the 
    database, using the Backup and RunScript tools or the SQL commands SCRIPT 
    and RUNSCRIPT.
</li><li>Server: In some situations, calling close() on a ResultSet or Statement threw 
    an exception. This has been fixed.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(16,
'New version available: 1.0 / 2006-11-20', '2006-11-20 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>New SQL statement SET SCHEMA to change the current schema of this session.
</li><li>New system function SCHEMA() to get the current schema.
</li><li>SCRIPT: New option BLOCKSIZE to split BLOB and CLOB data into separate blocks, to avoid OutOfMemory problems.
</li><li>CURRVAL and NEXTVAL functions: New optional sequence name parameter.
</li><li>The default cache size is now 65536 pages instead of 32768.
</li><li>New optimization to reuse subquery results. Can be disabled with SET OPTIMIZE_REUSE_RESULTS 0.
</li><li>EXPLAIN... results are now formatted on multiple lines so they are easier to read.
</li><li>The Spanish translation was completed by Miguel Angel. Thanks a lot! Translations to other languages are always welcome.
</li><li>The Recovery tool has been improved. It now creates a SQL script file that can be executed directly.
</li><li>LENGTH now returns the precision for CLOB, BLOB, and BINARY (and is therefore faster).
</li><li>The built-in FTP server can now access a virtual directory stored in a database.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>When using the READ_COMMITTED isolation level, a transaction now waits until there are no write locks.
</li><li>INSERT INTO ... SELECT ... and ALTER TABLE with CLOB and/or BLOB data did not work.
</li><li>CSV tool: the methods setFieldSeparatorWrite and setRowSeparatorWrite where not accessible.
</li><li>ALTER TABLE ADD did throw a strange message if the table contained views. Now the message is better, 
    but it is still not possible to do that if views on this table exist.
</li><li>ALTER TABLE: If there was a foreign key in another table that references to the change table, the constraint was dropped.
</li><li>Direct links to the Javadoc were not working.
</li><li>Inserting rows into linked tables did not work for HSQLDB when the value was NULL.
</li><li>Oracle SYNONYM tables are now listed in the H2 Console.
</li><li>CREATE LINKED TABLE didn''t work for Oracle SYNONYM tables.
</li><li>When using the server version, when not closing result sets or using nested DatabaseMetaData result sets, the connection could break.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(15,
'New version available: 1.0 / 2006-11-03', '2006-11-03 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>New SQL statement COMMENT ON ... IS ...
</li><li>Two simple full text search implementations (Lucene and native) are now included. 
    This is work in progress, and currently undocumented. 
    See test/org/h2/samples/fullTextSearch.sql and History for more details.
</li><li>Index names of referential constraints are now prefixed with the constraint name.
</li><li>Improved search functionality in the HTML documentation (highlight).
</li><li>Triggers can now be defined on a list of actions.
</li><li>On some systems (for example, some Linux VFAT and USB flash disk drivers), 
    RandomAccessFile.setLength does not work correctly. 
    A workaround for this problem has been implemented.
</li><li>DatabaseMetaData.getTableTypes now also returns SYSTEM TABLE and TABLE VIEW.
    Please tell me if this breaks other applications or tools.
</li><li>Java functions with Blob or Clob parameters are now supported.
</li><li>Added a ''remarks'' column to most system tables. 
</li><li>New system table INFORMATION_SCHEMA.TRIGGERS
</li><li>PostgreSQL compatibility: Support for the date format 2006-09-22T13:18:17.061
</li><li>MySQL compatibility: ResultSet.getString("PEOPLE.NAME") is now supported.
</li><li>JDBC 4.0 driver auto discovery: When using JDK 1.6, 
    Class.forName("org.h2.Driver") is no longer required.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>Wide b-tree indexes (with large VARCHAR columns for example) could get corrupted.
</li><li>If a custom shutdown hook was installed, and the database was called at shutdown, 
    a NullPointException was thrown. 
    Now, a error message with instructions how to fix this is thrown.
</li><li>If SHUTDOWN was called and DB_CLOSE_DELAY was set, the database was not closed.
</li><li>Subqueries with order by outside the column list didn''t work correctly.
</li><li>Linked Tables: Only the first column was linked when linking to PostgreSQL.
</li><li>Sequences: When the database is not closed normally, the value was not set correctly.
</li><li>The optimization for IN(SELECT...) was too aggressive.
</li><li>Blob.getBytes skipped the wrong number of bytes.
</li><li>Group by a function didn''t work if a column alias was specified in the select list.
</li><li>LOCK_MODE 0 (READ_UNCOMMITTED) did not work when using multiple connections.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(14,
'New version available: 1.0 / 2006-10-10', '2006-10-10 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>Support for DOMAIN (user defined data types).
</li><li>Support for UUID.
</li><li>Function aliases may now optionally include parameter classes.
</li><li>A small FTP server is now included (disabled by default).
</li><li>Can now compile everything with JDK 1.6 (however, only very few of the JDBC 4.0 features are implemented currently).
</li><li>The multi-threaded kernel can not be enabled using SET MULTI_THREADED 1.
    A new tests has been written for this feature, and additional synchronization has been added.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>Could not re-connect to a database when ALLOW_LITERALS or COMPRESS_LOB was set.
</li><li>Opening and closing connections in many threads sometimes failed.
</li><li>Reconnect didn''t work after renaming a user if rights were granted for this user.
</li><li>GROUP BY an formula or function didn''t work if the same expression was used in the select list.
</li><li>Redundant () in a IN subquery is now supported: where id in ((select id from test)).
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(13,
'New version available: 1.0 / 2006-09-24', '2006-09-24 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>Protection against SQL injection: New command SET ALLOW_LITERALS {NONE|ALL|NUMBERS}.
    With SET ALLOW_LITERALS NONE, SQL injections are not possible because literals in SQL statements are rejected;
    User input must be set using parameters (''?'') in this case.
</li><li>New concept ''Constants'': New SQL statements CREATE CONSTANT and DROP CONSTANT.
</li><li>CREATE TABLE ... AS SELECT ... is now supported.
</li><li>New data type OTHER (alternative names OBJECT and JAVA_OBJECT). When using this data type,
    Java Objects are automatically serialized and deserialized in the JDBC layer.
</li><li>Improved performance for MetaData calls
</li><li>Improved BatchUpdateException
</li><li>DatabaseMetaData.getProcedures and getProcedureColumns are implemented now
</li><li>An exception is now thrown on unknown setting in the database URL (which are most likely typos)
</li><li>The log size is now automatically increased to at least 10% of the data file.
</li><li>Backup and Runscript tools now support options (for compression and so on)
</li><li>InetAddress.getByName("127.0.0.1")  instead of InetAddress.getLocalHost() is now used to get the loopback address
</li><li>CREATE SCHEMA: The authorization part is now optional.
</li><li>DROP TABLE: Can now drop more than one column in one step: DROP TABLE A, B
</li><li>LOBs are now automatically converted to files by default.
</li><li> Optimizations for WHERE ... IN(...) and SELECT MIN(..), MAX(..) are now enabled by default.
</li><li>New system function LOCK_MODE()
</li><li>Connection.setTransactionIsolation and getTransactionIsolation now set / get the LOCK_MODE of the database.
</li><li>New LOCK_MODE 3 (READ_COMMITTED). Table level locking, but only when writing (no read locks).
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>Wide b-tree indexes (with large VARCHAR columns for example) with a long common prefix (where many
    rows start with the same text) could get corrupted. Fixed.
</li><li>Reading from compressed LOBs didn''t work in some cases. Fixed.
</li><li>CLOB / BLOB: Copying LOB data directly from one table to another, and altering a table with with LOBs did not work,
    the BLOB data was deleted when the old table was deleted. Fixed.
</li><li>[NOT] EXISTS(SELECT ... EXCEPT SELECT ...) did not work in all cases. Fixed.
</li><li>Functions returning a result set are now called as documented.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

INSERT INTO ITEM VALUES(12,
'New version available: 1.0 / 2006-09-10', '2006-09-10 12:00:00',
'A new version of H2 is available for <a href="http://www.h2database.com">download</a>.
<br />
<b>Changes and new functionality:</b>
<ul>
<li>SET IGNORECASE is now supported for compatibility with HSQLDB.
</li><li>New SQL statement DROP ALL OBJECTS [DELETE FILES] to drop all tables, sequences and so on.
</li><li>Improved OpenOffice compatibility.
</li><li>New setting SET COMPRESS_LOB {NO|LZF|DEFLATE} to automatically compress BLOB and CLOB data.
</li><li>The script can now be compressed. Syntax: SCRIPT TO ''file'' COMPRESSION {DEFLATE|LZF|ZIP|GZIP}.
</li><li>Now an exception is thrown when the an overflow occurs for mathematical operations (sum, multiply and so on) for the data type selected.
    This was implemented in the previous version but is now enabled by default.
</li><li>Updated the performance test so that Firebird can be tested as well. Results are not included currently,
    information how to best test Firebird should be sent to the support address or posted in the Forum.
</li></ul>
<b>Bugfixes:</b>
<ul>
<li>ORDER BY an expression didn''t work when using GROUP BY at the same time.
</li><li>A problem with referential constraints in the SCRIPT file has been fixed.
</li><li>Console: The setting ;hsqldb.default_table_type=cached was added to the H2 database instead of the HSQLDB database.
</li><li>Docs: The cross references in the SQL grammar docs where broken in the last release.
</li><li>Deleting many rows from a table with a self-referencing constraint with ''on delete cascade'' did not work.
</li><li>ROWNUM didn''t always work as expected when using subqueries.
</li><li>Correlated subqueries: It is now possible to use columns of the outer query in the select list of the inner query.
</li></ul>
For future plans, see the new ''Roadmap'' page on the web site.
');

SELECT 'newsfeed-rss.xml' FILE,
    XMLSTARTDOC() ||
    XMLNODE('rss', XMLATTR('version', '2.0'),
        XMLNODE('channel', NULL,
            XMLNODE('title', NULL, C.TITLE) ||
            XMLNODE('link', NULL, C.LINK) ||
            XMLNODE('description', NULL, C.DESC) ||
            XMLNODE('language', NULL, C.LANGUAGE) ||
            XMLNODE('pubDate', NULL, FORMATDATETIME(C.PUB, 'EEE, d MMM yyyy HH:mm:ss z', 'en', 'GMT')) ||
            XMLNODE('lastBuildDate', NULL, FORMATDATETIME(C.LAST, 'EEE, d MMM yyyy HH:mm:ss z', 'en', 'GMT')) ||
            GROUP_CONCAT(
                XMLNODE('item', NULL,
                    XMLNODE('title', NULL, I.TITLE) ||
                    XMLNODE('link', NULL, C.LINK) ||
                    XMLNODE('description', NULL, XMLCDATA(I.TITLE))
                )
            ORDER BY I.ID DESC SEPARATOR '')
        )
    ) CONTENT
FROM CHANNEL C, ITEM I
UNION
SELECT 'newsfeed-atom.xml' FILE,
    XMLSTARTDOC() ||
    XMLNODE('feed', XMLATTR('version', '0.3') || XMLATTR('xmlns', 'http://purl.org/atom/ns#') || XMLATTR('xml:lang', C.LANGUAGE),
        XMLNODE('title', XMLATTR('type', 'text/plain') || XMLATTR('mode', 'escaped'), C.TITLE) ||
        XMLNODE('author', NULL, XMLNODE('name', NULL, C.AUTHOR)) ||
        XMLNODE('link', XMLATTR('rel', 'alternate') || XMLATTR('type', 'text/html') || XMLATTR('href', C.LINK), NULL) ||
        XMLNODE('modified', NULL, FORMATDATETIME(C.LAST, 'yyyy-MM-dd''T''HH:mm:ss.SSS', 'en', 'GMT')) ||
        GROUP_CONCAT(
            XMLNODE('entry', NULL,
                XMLNODE('title', XMLATTR('type', 'text/plain') || XMLATTR('mode', 'escaped'), I.TITLE) ||
                XMLNODE('link', XMLATTR('rel', 'alternate') || XMLATTR('type', 'text/html') || XMLATTR('href', C.LINK), NULL) ||
                XMLNODE('id', NULL, XMLTEXT(C.LINK || '/' || I.ID)) ||
                XMLNODE('issued', NULL, FORMATDATETIME(I.ISSUED, 'yyyy-MM-dd''T''HH:mm:ss.SSS', 'en', 'GMT')) ||
                XMLNODE('modified', NULL, FORMATDATETIME(I.ISSUED, 'yyyy-MM-dd''T''HH:mm:ss.SSS', 'en', 'GMT')) ||
                XMLNODE('content', XMLATTR('type', 'text/html') || XMLATTR('mode', 'escaped'), XMLCDATA(I.DESC))
            )
        ORDER BY I.ID DESC SEPARATOR '')
    ) CONTENT
FROM CHANNEL C, ITEM I
UNION
SELECT '-newsletter-' FILE, I.DESC CONTENT FROM ITEM I WHERE I.ID = (SELECT MAX(ID) FROM ITEM)